//---------------------------------------------------------------------------------------
// Gandalf Graphix Cool Static aka Label - HotTracking effect, Transparency, Pushable Eff
// (C)2001 Alexander Turkovsky
//---------------------------------------------------------------------------------------

#include "StdAfx.h"
#include "GGLabel.h"

//---------------------------------------------------------------------------------------
IMPLEMENT_DYNAMIC( GGLabel, CStatic )

//---------------------------------------------------------------------------------------
BEGIN_MESSAGE_MAP( GGLabel, CStatic )
	//{{AFX_MSG_MAP(GGLabel)
	   ON_WM_ENABLE()
	   ON_WM_PAINT()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


//---------------------------------------------------------------------------------------
GGLabel::GGLabel( void )
{
   m_NormalColor = RGB( 0, 0, 0 );
   m_HiliteColor = RGB( 0, 0, 120 );
   m_PressedColor = RGB( 120, 0, 0 );
   m_LabelState = GGLABEL_STATE_NORMAL;
   m_Captured = false;
   m_SimulatePress = false;

   // m_HotTracking = true;
   // m_IsPushable = true;
};

//---------------------------------------------------------------------------------------
void GGLabel::OnEnable( BOOL bEnable ) 
{
   CStatic::OnEnable( bEnable );
   RedrawWindow();
}

//---------------------------------------------------------------------------------------
void GGLabel::OnPaint()
{
   //CStatic::OnPaint();
   char szText[ 300 ];
   PAINTSTRUCT ps;
   RECT rc;

   CStatic::GetWindowText( szText, 300 );
   CStatic::GetWindowRect( &rc );
   ScreenToClient( &rc );

   if( !szText[ 0 ] || rc.left == rc.right || rc.top == rc.bottom )
      // ... Nothing to do : return immediately
      return;

   DWORD nTextAlignment = GetStyle() & ( SS_LEFT | SS_CENTER | SS_RIGHT );
   if( !nTextAlignment ) nTextAlignment = SS_LEFT;
   UINT nFormat = ( ( GetStyle() & SS_LEFTNOWORDWRAP ) == SS_LEFTNOWORDWRAP ? DT_SINGLELINE : DT_WORDBREAK ) | 
                    ( nTextAlignment == SS_LEFT ? DT_LEFT : ( nTextAlignment == SS_RIGHT ? DT_RIGHT : DT_CENTER ) );// | DT_VCENTER;
   
   // szText[ 0 ] = 0;
   // if( nFormat & DT_WORDBREAK ) lstrcat( szText, "DT_WORDBREAK, " );
   // if( nFormat & DT_SINGLELINE ) lstrcat( szText, "DT_SINGLELINE, " );
   // if( nFormat & DT_LEFT ) lstrcat( szText, "DT_LEFT, " );
   // if( nFormat & DT_RIGHT ) lstrcat( szText, "DT_RIGHT, " );
   // if( nFormat & DT_CENTER ) lstrcat( szText, "DT_CENTER, " );
   // if( nFormat & DT_VCENTER ) lstrcat( szText, "DT_VCENTER, " );

   // DT_WORDBREAK, DT_SINGLELINE, DT_LEFT, DT_RIGHT, DT_CENTER, DT_VCENTER

   // get device handler
   CDC* cdc = CStatic::BeginPaint( &ps );

   // Set font
   CFont *pcf = CStatic::GetFont();
   cdc->SelectObject( pcf );
   
   // set label as transparent
   cdc->SetBkMode( TRANSPARENT );
   
   // set label color
   COLORREF labclr = m_NormalColor;
   if( m_LabelState == GGLABEL_STATE_HILITE ) labclr = m_HiliteColor;
   if( m_LabelState == GGLABEL_STATE_PRESSED ) labclr = m_PressedColor;

   //char ts[300];
   //wsprintf( ts, ", %Xh", GetTickCount() );
   //lstrcat( szText, ts );
   
   // check for enable
   if( !IsWindowEnabled() ) labclr = GetSysColor( COLOR_GRAYTEXT );
   cdc->SetTextColor( labclr );
   
   // draw text
   cdc->DrawText( szText, lstrlen( szText ), &rc, nFormat );

   // done
   EndPaint( &ps );
}

//---------------------------------------------------------------------------------------
LRESULT GGLabel::DefWindowProc( UINT Msg, WPARAM wParam, LPARAM lParam )
{
   
   if( Msg == WM_MOUSEMOVE )
   {
      if( IsWindowEnabled() )
      {
         POINTS pts = MAKEPOINTS( lParam );
         RECT rc;
         GetWindowRect( &rc );
         ScreenToClient( &rc );
         
         bool MouseInside = false;
         if( pts.x > rc.left && pts.x < rc.right &&
             pts.y > rc.top && pts.y < rc.bottom ) MouseInside = true;

         if( m_Captured )
         {

            if( MouseInside )
            {
               // inside
               if( m_LabelState != GGLABEL_STATE_PRESSED )
               {
                  m_LabelState = GGLABEL_STATE_PRESSED;
                  RedrawWindow();
               }
            }else{
               // outside
               if( m_LabelState != GGLABEL_STATE_NORMAL )
               {
                  m_LabelState = GGLABEL_STATE_NORMAL;
                  RedrawWindow();
               }
            }
         }else{
            // hilite text
            if( m_HotTracking )
            {
               if( m_LabelState != GGLABEL_STATE_HILITE )
               {
                  m_LabelState = GGLABEL_STATE_HILITE;
                  RedrawWindow();
               }
               // start check up timer
               SetTimer( GGLABEL_CHECKTIMER_ID, 150, NULL );
            }
         }
      }
      //MessageBox( "mousemove" );
   }
   
   if( Msg == WM_TIMER )
   {
      if( wParam == GGLABEL_CHECKTIMER_ID && !m_Captured )
      {
         // check for inside
         POINT pt;
         GetCursorPos( &pt );
         RECT rc;
         GetWindowRect( &rc );
         //ScreenToClient( &rc );

         if( pt.x < rc.left || pt.x > rc.right ||
             pt.y < rc.top || pt.y > rc.bottom )
         {
            KillTimer( GGLABEL_CHECKTIMER_ID );
            // set normal state
            m_LabelState = GGLABEL_STATE_NORMAL;
            RedrawWindow();
         }
      }
   }

   if( Msg == WM_LBUTTONDOWN )
   {
      if( !m_SimulatePress )
      {
         if( m_IsPushable )
         {
            m_LabelState = GGLABEL_STATE_PRESSED;
            RedrawWindow();
            SetCapture();// capture the mouse input while user holding lbtn
            m_Captured = true;
         }
         return true;// disallow instant static OnClick (OnClick called before button up)
      }
   }
   
   if( Msg == WM_LBUTTONUP )
   {
      if( m_IsPushable )
      {
         m_LabelState = GGLABEL_STATE_NORMAL;
         RedrawWindow();
         ReleaseCapture();// user up lbtn
         m_Captured = false;
      }
      // simulate OnClicked if mouse inside label

      // check for inside
      POINT pt;
      GetCursorPos( &pt );
      RECT rc;
      GetWindowRect( &rc );

      if( pt.x > rc.left && pt.x < rc.right &&
          pt.y > rc.top && pt.y < rc.bottom )
      {
         // simulate
         m_SimulatePress = true;
         CStatic::SendMessage( WM_LBUTTONDOWN, wParam, lParam );
         m_SimulatePress = false;
      }
   }
   
   if( Msg == WM_SETTEXT )
   {
      RedrawWindow();
   }
   
   if( Msg == WM_DESTROY )
   {
      KillTimer( GGLABEL_CHECKTIMER_ID );
   }

   return CStatic::DefWindowProc( Msg, wParam, lParam );
}

//---------------------------------------------------------------------------------------
void GGLabel::SetNormalColor( COLORREF Clr )
{
   m_NormalColor = Clr;
   RedrawWindow();
}

//---------------------------------------------------------------------------------------
void GGLabel::SetHiliteColor( COLORREF Clr )
{
   m_HiliteColor = Clr;
   RedrawWindow();
}

//---------------------------------------------------------------------------------------
void GGLabel::SetPressedColor( COLORREF Clr )
{
   m_PressedColor = Clr;
   RedrawWindow();
}

//---------------------------------------------------------------------------------------
COLORREF GGLabel::GetNormalColor( void )
{
   return m_NormalColor;
}

//---------------------------------------------------------------------------------------
COLORREF GGLabel::GetHiliteColor( void )
{
   return m_HiliteColor;
}

//---------------------------------------------------------------------------------------
COLORREF GGLabel::GetPressedColor( void )
{
   return m_PressedColor;
}

//---------------------------------------------------------------------------------------
void GGLabel::SetHotTracking( bool HotTracking )
{
   m_HotTracking = HotTracking;
   RedrawWindow();
}

//---------------------------------------------------------------------------------------
void GGLabel::SetPushability( bool Pushability )
{
   m_IsPushable = Pushability;
   RedrawWindow();
}

//---------------------------------------------------------------------------------------
bool GGLabel::GetHotTracking( void )
{
   return m_HotTracking;
}

//---------------------------------------------------------------------------------------
bool GGLabel::GetPushability( void )
{
   return m_IsPushable;
}

//---------------------------------------------------------------------------------------
void GGLabel::SetEffectOptions( bool HotTracking, bool Pushability )
{
   m_HotTracking = HotTracking;
   m_IsPushable = Pushability;
   RedrawWindow();
}

//---------------------------------------------------------------------------------------
void GGLabel::SetColors( COLORREF NormalClr, COLORREF HiliteClr, COLORREF PushClr )
{
   m_NormalColor = NormalClr;
   m_HiliteColor = HiliteClr;
   m_PressedColor = PushClr;
   RedrawWindow();
}
